@@ -3,6 +3,7 @@ weightitMSM <- function(formula.list, data = NULL, method = "ps", stabilize = FA
                         verbose = FALSE, include.obj = FALSE, is.MSM.method, weightit.force = FALSE, ...) {
 
   A <- list(...)
+  by.name <- paste(deparse(substitute(by)), collapse = "")
 
   call <- match.call()
 
@@ -101,7 +102,7 @@ weightitMSM <- function(formula.list, data = NULL, method = "ps", stabilize = FA
     # w.data <- data.frame(treat.list[[i]], covs.list[[i]])
     # w.formula <- formula(w.data)
 
-    processed.by <- process.by(by = by, data = data,
+    processed.by <- process.by(by.name, data = data,
                                        treat = treat.list[[i]],
                                        treat.name = treat.name)
 
@@ -118,7 +119,7 @@ weightitMSM <- function(formula.list, data = NULL, method = "ps", stabilize = FA
       obj <- do.call("weightit.fit", c(list(covs = covs.list,
                                             treat = treat.list,
                                             s.weights = s.weights,
-                                            by.factor = processed.by[["by.factor"]],
+                                            by.factor = attr(processed.by, "by.factor"),
                                             stabilize = stabilize,
                                             method = method,
                                             moments = moments,
@@ -154,7 +155,7 @@ weightitMSM <- function(formula.list, data = NULL, method = "ps", stabilize = FA
                                               treat = treat.list[[i]],
                                               treat.type = get.treat.type(treat.list[[i]]),
                                               s.weights = s.weights,
-                                              by.factor = processed.by[["by.factor"]],
+                                              by.factor = attr(processed.by, "by.factor"),
                                               estimand = estimand,
                                               focal = NULL,
                                               stabilize = FALSE,
@@ -196,7 +197,7 @@ weightitMSM <- function(formula.list, data = NULL, method = "ps", stabilize = FA
                                                    treat = treat.list[[i]],
                                                    treat.type = get.treat.type(treat.list[[i]]),
                                                    s.weights = s.weights,
-                                                   by.factor = processed.by[["by.factor"]],
+                                                   by.factor = attr(processed.by, "by.factor"),
                                                    estimand = estimand,
                                                    focal = NULL,
                                                    stabilize = FALSE,
@@ -241,12 +242,14 @@ weightitMSM <- function(formula.list, data = NULL, method = "ps", stabilize = FA
               ps.list = ps.list,
               s.weights = s.weights,
               #discarded = NULL,
-              by = processed.by[["by.components"]],
+              by = processed.by,
               call = call,
               stabilization = stabout,
               obj = obj.list
   )
 
+  out <- clear_null(out)
+
   class(out) <- c("weightitMSM", "weightit")
 
   return(out)
